package vc.thinker.mqtt.proxyclient.actor.down;

import akka.actor.AbstractActor;
import akka.japi.pf.ReceiveBuilder;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;
import vc.thinker.absctacts.mqtt.entity.MqttResponse;
import vc.thinker.absctacts.protocolproxy.dto.MeterInfo;
import vc.thinker.absctacts.protocolproxy.protocol.IThirdPartyProtocolSyncMeterInfo;
import vc.thinker.mqtt.proxyclient.entity.down.MeterChange;
import vc.thinker.mqtt.proxyclient.entity.down.PlateFormMeterInfo;
import vc.thinker.mqtt.proxyclient.enums.ThirdPartyProxyTopicDefinition;

import javax.annotation.Resource;
import java.util.List;

/**
 * 仪表改变通知
 *
 * @author HeTongHao
 * @since 2022/18/19 15:43
 */
@Slf4j
@Component
@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class MeterChangeActor extends AbstractActor {
    @Resource
    private List<IThirdPartyProtocolSyncMeterInfo> thirdPartyProtocols;

    public MeterChangeActor() {
        receive(ReceiveBuilder.match(MqttResponse.class, mqttResponse -> {
            String topic = mqttResponse.getTopic();
            String protocolType = ThirdPartyProxyTopicDefinition.resolveProtocolCodeByTopic(topic);
            for (IThirdPartyProtocolSyncMeterInfo thirdPartyProtocol : thirdPartyProtocols) {
                if (protocolType.equals(thirdPartyProtocol.protocolType().getCode())) {
                    MeterChange meterChange = JSONObject.parseObject(mqttResponse.getMessage(), MeterChange.class);
                    PlateFormMeterInfo meterInfo = meterChange.getMeterInfo();
                    // 发送到具体协议去执行
                    switch (meterChange.getOperateType()) {
                        case "PUT":
                            thirdPartyProtocol.putMeter(MeterInfo
                                    .builder()
                                    .id(meterInfo.getId())
                                    .code(meterInfo.getCode())
                                    .build()
                            );
                            break;
                        case "DELETE":
                            thirdPartyProtocol.deleteMeter(meterInfo.getId());
                            break;
                        default:
                            log.error("不支持操作:" + meterChange.getOperateType());
                    }
                    break;
                }
            }
        }).build());
    }
}
